#ifndef VARIABLE_H
#define VARIABLE_H

#include <opencv4/opencv2/core.hpp>
#include <opencv4/opencv2/imgcodecs.hpp>
#include <opencv4/opencv2/imgproc.hpp>
#include "../codec/kylinimagecodec/kylinimagecodec.h"
#include <QString>
#include <QGSettings>
#include <QDir>
#include <QSize>
#include <QPixmap>
#include <QStandardPaths>
#include <QDebug>
#include <unistd.h>

struct ImageAndInfo
{
    QFileInfo info; //信息
    QPixmap image; //图片
    int imageNumber; //标签
    int proportion; //比例
    QString colorSpace;//图片空间
    QString imageSize;//图片尺寸
    bool openSuccess = true;//判断打开原图是否成功，默认为true，打开成功
};
struct cutBoxColor
{
    QPoint startPostion;//--备用--起点
    QPoint endPostion;
    bool colorChange = true;//判断是黑变白还是白变黑，默认黑变白-true
    int key = 50;//透明度
};
struct CutRegion{
    QPointF startPostion;//起点
    int width = 1;//裁剪宽度，最小宽度1
    int height = 1;//裁剪高度，最小高度1
};
Q_DECLARE_METATYPE(ImageAndInfo)//QVarant 注册自定义类型
Q_DECLARE_METATYPE(cutBoxColor)//QVarant 注册自定义类型

namespace CutMinValue {
constexpr int minValue = 1;
}
class Variable
{

public:
    static const QString PROGRAM_NAME; //项目名称
    static const QString PHOTO_VIEW_DBUS_SERVICENAME; //DBUS服务名称
    static const QString PHOTO_VIEW_GSETTINGS_SERVICENAME; //QGSettings服务名称
    static const QString TEMP_PATH; //临时文件路径
    static const QString PHOTO_VIEW_DBUS_PARH; //DBUS路径
    static const QString PHOTO_VIEW_DBUS_INTERFACE;//DBUS接口
    static const QPair<QString,QString> LOG_PATH; //日志路径
    static QtMsgType g_logLevel; //日志级别
    static bool g_logToFile; //日志写入文件
    static const qint64 MAX_LOG_SIZE;//最大日志文件大小
    static const QMap<QString,QString> SUPPORT_CMD; //支持的命令列表
    static const QStringList SUPPORT_FORMATS; //支持的格式列表
    static const QString SUPPORT_FORMATS_CLASSIFY; //支持的格式列表分类
    static const QStringList BACKGROUND_SUPPORT_FORMATS; //壁纸支持的格式列表
    static const QStringList SUPPORT_FORMATS_END; //
    static const QString SUPPORT_FORMATS_SAVEAS;//另存为格式
    static const QStringList SUPPORT_FORMATS_NOT;//另存为格式-暂时不支持别的格式保存为此些格式
    static const QSize ALBUM_IMAGE_SIZE; //相册缩略图尺寸
    static const QSize NAVIGATION_SIZE; //导航器尺寸
    static const int PICTURE_DEEPEN_KEY; //图片加深值
    static const int REFRESH_RATE; //刷新间隔
    static const int RESIZE_KEY; //每次放大缩小的值
    static const int RESIZE_KEY_MAX; //能够放大的最大值
    static const int RESIZE_KEY_MIN; //能够缩小的最小值
    static const int DEFAULT_MOVIE_TIME_INTERVAL; //默认动图时间间隔
    static const QString API_TYPE; //是否为外部作为API调用的标识
    static const QStringList freeimageCanSupportFormats;//freeimage支持的格式列表
    static QString g_themeStyle;//主题切换
    static QStringList g_damagedImage;//存所有损坏图片路径
    static void setSettings(const QString &key,const QVariant &vlue);//写入配置文件
    static QVariant getSettings(const QString &key);//读取配置文件
    static const int LOADIMAGE_NUM;//一次加载的数量
    static const int DEFAULT_WHITE_COLOR;//绘制的默认白色色值
    static const int DEFAULT_BLACK_COLOR;//绘制的默认黑色色值
    static const QString DAMAGED_IMAGE_BLACK;//深色模式下的损坏图片路径
    static const QString DAMAGED_IMAGE_WHITE;//浅色模式下的损坏图片路径
private:
    static const QStringList opencvCanSupportFormats; //opencv支持的格式列表
    static const QStringList opencvCannotSupportFormats; //opencv不支持的格式列表
    static QStringList creatSupportFormats();//所有支持格式
    static QGSettings *m_settings;//配置文件对象
    static QGSettings *getSettings();//初始化gsettings
    static const QString creatTempPath();//创建临时目录
    static QMap<QString,QString> getSupportCmd();//创建支持的命令列表
    static void onGsettingChange(const QString &key);//监听gsettings改变槽函数
    static QPair<QString,QString> getLogPath();//获取日志目录
    static QtMsgType getLogLevel();//获取日志级别

};

#endif // VARIABLE_H
